import { NgModule, ɵɵdefineInjectable, ɵɵinject, Injectable, Inject } from '@angular/core';
import { StorageServiceModule, LOCAL_STORAGE } from 'ngx-webstorage-service';
import { VgCoreModule } from '@videogular/ngx-videogular/core';
import { VgControlsModule } from '@videogular/ngx-videogular/controls';
import { VgOverlayPlayModule } from '@videogular/ngx-videogular/overlay-play';
import { VgBufferingModule } from '@videogular/ngx-videogular/buffering';
import { of } from 'rxjs';
import Shepherd from 'shepherd.js';
import { NavigationEnd, Router } from '@angular/router';

class UserOnboardingLibModule {
}
UserOnboardingLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    StorageServiceModule,
                    VgCoreModule,
                    VgControlsModule,
                    VgOverlayPlayModule,
                    VgBufferingModule
                ],
                exports: []
            },] }
];

class SaveSCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const newTourState = this.parameters.state;
        this.storage.set(newTourState.sessionId, newTourState);
        return of(newTourState);
    }
}
class LoadSCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const currentState = this.storage.get(this.parameters.sessionId);
        return of(currentState);
    }
}

class SaveTCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const newTour = {
            tourId: this.parameters.tourId,
            tourSteps: this.parameters.tourSteps,
            styleSheet: this.parameters.styleSheet,
        };
        this.storage.set(this.parameters.tourId, newTour);
        return of(newTour);
    }
}
class LoadTCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const existingTour = this.storage.get(this.parameters.tourId);
        return of(existingTour);
    }
}

class TourStoreServiceService {
    constructor(storage) {
        this.storage = storage;
        this.commandMap = new Map();
        this.functionMap = new Map();
        this.defaultSSCommand = new SaveSCommand(this.storage);
        this.defaultSTCommand = new SaveTCommand(this.storage);
        this.defaultLSCommand = new LoadSCommand(this.storage);
        this.defaultLTCommand = new LoadTCommand(this.storage);
        this.commandMap.set('SaveTourCommand', this.defaultSTCommand);
        this.commandMap.set('LoadTourCommand', this.defaultLTCommand);
        this.commandMap.set('SaveStateCommand', this.defaultSSCommand);
        this.commandMap.set('LoadStateCommand', this.defaultLSCommand);
    }
    registerSaveTourCommand(cmd) {
        this.commandMap.set('SaveTourCommand', cmd);
    }
    registerLoadTourCommand(cmd) {
        this.commandMap.set('LoadTourCommand', cmd);
    }
    registerSaveStateCommand(cmd) {
        this.commandMap.set('SaveStateCommand', cmd);
    }
    registerLoadStateCommand(cmd) {
        this.commandMap.set('LoadStateCommand', cmd);
    }
    saveTour(parameters) {
        const command = this.commandMap.get('SaveTourCommand');
        command.parameters = parameters;
        return command.execute();
    }
    loadTour(parameters) {
        const command = this.commandMap.get('LoadTourCommand');
        command.parameters = parameters;
        return command.execute();
    }
    saveState(parameters) {
        const command = this.commandMap.get('SaveStateCommand');
        command.parameters = parameters;
        return command.execute();
    }
    loadState(parameters) {
        const command = this.commandMap.get('LoadStateCommand');
        command.parameters = parameters;
        return command.execute();
    }
    registerFnRef(key, fn) {
        this.functionMap.set(key, fn);
    }
    getFnByKey(key) {
        return this.functionMap.get(key);
    }
    createUUID() {
        let dt = new Date().getTime();
        const num1 = 16;
        const num2 = 0x3;
        const num3 = 0x8;
        const crypto = window.crypto || window.Crypto.prototype;
        const array = new Uint32Array(1);
        crypto.getRandomValues(array); // Compliant for security-sensitive use cases
        const FileId = array[0];
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, x => {
            const r = (dt + FileId * num1) % num1 || 0;
            dt = Math.floor(dt / num1);
            return (x === 'x' ? r : (r && num2) || num3).toString(num1);
        });
    }
    generateSessionId() {
        this.sessionId = this.createUUID();
        this.storage.set('TOUR_SESSION_ID', this.sessionId);
    }
    getSessionId() {
        return this.storage.get('TOUR_SESSION_ID');
    }
}
TourStoreServiceService.ɵprov = ɵɵdefineInjectable({ factory: function TourStoreServiceService_Factory() { return new TourStoreServiceService(ɵɵinject(LOCAL_STORAGE)); }, token: TourStoreServiceService, providedIn: "root" });
TourStoreServiceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
TourStoreServiceService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [LOCAL_STORAGE,] }] }
];

class TourServiceService {
    constructor(tourStoreService, router) {
        this.tourStoreService = tourStoreService;
        this.router = router;
        this.interval = 100;
        this.tour = new Shepherd.Tour({
            defaultStepOptions: {
                cancelIcon: {
                    enabled: true,
                },
                classes: 'class-1 class-2',
                scrollTo: { behavior: 'smooth', block: 'center' },
            },
        });
    }
    addRemovedSteps(removedSteps) {
        const count = removedSteps.length;
        for (let i = 0; i < count; i++) {
            this.tour.steps.splice(0, 0, this.tour.steps.pop());
        }
    }
    actionAssignment(e, b, wrapperNormalNext, wrapperNormalPrev, wrapperNext, wrapperPrev, func) {
        if (b.key === 'prevAction') {
            b.action =
                e.prevRoute === e.currentRoute ? wrapperNormalPrev : wrapperPrev;
        }
        else if (b.key === 'nextAction') {
            b.action =
                e.nextRoute === e.currentRoute ? wrapperNormalNext : wrapperNext;
        }
        else {
            b.action = func;
        }
    }
    navigationCheckNext(event, e) {
        if (event instanceof NavigationEnd && event.url === e.currentRoute) {
            this.waitForElement(e.attachTo.element, 0)
                .then(() => {
                this.tour.cancel();
                this.tour.next();
            })
                .catch(() => {
                throw new Error('Error detected in loading');
            });
        }
    }
    navigationCheckBack(event, e) {
        if (event instanceof NavigationEnd && event.url === e.currentRoute) {
            this.waitForElement(e.attachTo.element, 0)
                .then(() => {
                this.tour.cancel();
                this.tour.back();
            })
                .catch(() => {
                throw new Error('Error detected in loading');
            });
        }
    }
    waitForElement(querySelector, timeout = 0) {
        const startTime = new Date().getTime();
        return new Promise((resolve, reject) => {
            const timer = setInterval(() => {
                const now = new Date().getTime();
                if (document.querySelector(querySelector)) {
                    clearInterval(timer);
                    resolve();
                }
                else if (timeout && now - startTime >= timeout) {
                    clearInterval(timer);
                    reject();
                }
                else {
                    // do nothing
                }
            }, this.interval);
        });
    }
    triggerTour(tourInstance) {
        let removedSteps;
        const sessionId = this.tourStoreService.getSessionId();
        this.tourStoreService
            .loadState({ tourId: tourInstance.tourId, sessionId })
            .subscribe(currentStep => {
            if (currentStep) {
                let f = true;
                removedSteps = tourInstance.tourSteps.filter(e => {
                    if (e.id === currentStep.step || !f) {
                        f = false;
                    }
                    return f;
                });
                let flag = false;
                tourInstance.tourSteps = tourInstance.tourSteps.filter(e => {
                    if (e.id === currentStep.step || flag) {
                        flag = true;
                    }
                    return flag;
                });
            }
            else {
                this.tourStoreService.generateSessionId();
            }
            tourInstance.tourSteps.forEach(e => {
                e.buttons.forEach(b => {
                    const key = b.key;
                    const func = this.tourStoreService.getFnByKey(key);
                    const wrapperNext = () => {
                        this.tourStoreService.saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.nextStepId,
                            },
                        });
                        this.router.navigate([e.nextRoute]);
                        this.router.events.subscribe((event) => {
                            const nextStep = tourInstance.tourSteps.filter(ts => ts.id === e.nextStepId)[0];
                            this.navigationCheckNext(event, nextStep);
                        });
                    };
                    const wrapperPrev = () => {
                        this.tourStoreService.saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.prevStepId,
                            },
                        });
                        this.router.navigate([e.prevRoute]);
                        this.router.events.subscribe((event) => {
                            const prevStep = tourInstance.tourSteps.filter(ts => ts.id === e.prevStepId)[0];
                            this.navigationCheckBack(event, prevStep);
                        });
                    };
                    const wrapperNormalNext = () => {
                        this.tourStoreService.saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.nextStepId,
                            },
                        });
                        this.tour.cancel();
                        this.tour.next();
                    };
                    const wrapperNormalPrev = () => {
                        this.tourStoreService.saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.prevStepId,
                            },
                        });
                        this.tour.cancel();
                        this.tour.back();
                    };
                    this.actionAssignment(e, b, wrapperNormalNext, wrapperNormalPrev, wrapperNext, wrapperPrev, func);
                });
            });
            this.tour.addSteps(tourInstance.tourSteps);
            this.tour.start();
            if (removedSteps !== undefined) {
                removedSteps.forEach(er => {
                    er.buttons.forEach(br => {
                        const k = br.key;
                        const funcRemoved = this.tourStoreService.getFnByKey(k);
                        const wrapperNextRemoved = () => {
                            this.tourStoreService.saveState({
                                tourId: tourInstance.tourId,
                                state: {
                                    sessionId: this.tourStoreService.getSessionId(),
                                    step: er.nextStepId,
                                },
                            });
                            this.router.navigate([er.nextRoute]);
                            this.router.events.subscribe((event) => {
                                this.navigationCheckNext(event, er);
                            });
                        };
                        const wrapperPrevRemoved = () => {
                            this.tourStoreService.saveState({
                                tourId: tourInstance.tourId,
                                state: {
                                    sessionId: this.tourStoreService.getSessionId(),
                                    step: er.prevStepId,
                                },
                            });
                            this.router.navigate([er.prevRoute]);
                            this.router.events.subscribe((event) => {
                                this.navigationCheckBack(event, er);
                            });
                        };
                        const wrapperNormalNextRemoved = () => {
                            this.tourStoreService.saveState({
                                tourId: tourInstance.tourId,
                                state: {
                                    sessionId: this.tourStoreService.getSessionId(),
                                    step: er.nextStepId,
                                },
                            });
                            this.tour.cancel();
                            this.tour.next();
                        };
                        const wrapperNormalPrevRemoved = () => {
                            this.tourStoreService.saveState({
                                tourId: tourInstance.tourId,
                                state: {
                                    sessionId: this.tourStoreService.getSessionId(),
                                    step: er.prevStepId,
                                },
                            });
                            this.tour.cancel();
                            this.tour.back();
                        };
                        this.actionAssignment(er, br, wrapperNormalNextRemoved, wrapperNormalPrevRemoved, wrapperNextRemoved, wrapperPrevRemoved, funcRemoved);
                    });
                });
                this.tour.addSteps(removedSteps);
                this.addRemovedSteps(removedSteps);
            }
        });
    }
    run(tourId) {
        this.tourStoreService.loadTour({ tourId }).subscribe(tourInstance => {
            if (tourInstance) {
                this.triggerTour(tourInstance);
            }
            else {
                throw new Error('Tour does not exist');
            }
        });
    }
}
TourServiceService.ɵprov = ɵɵdefineInjectable({ factory: function TourServiceService_Factory() { return new TourServiceService(ɵɵinject(TourStoreServiceService), ɵɵinject(Router)); }, token: TourServiceService, providedIn: "root" });
TourServiceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
TourServiceService.ctorParameters = () => [
    { type: TourStoreServiceService },
    { type: Router }
];

const prefix = `<vg-player><vg-overlay-play></vg-overlay-play><vg-buffering></vg-buffering>
<vg-scrub-bar><vg-scrub-bar-current-time></vg-scrub-bar-current-time><vg-scrub-bar-buffering-time></vg-scrub-bar-buffering-time></vg-scrub-bar>
<vg-controls><vg-play-pause></vg-play-pause> <vg-playback-button></vg-playback-button><vg-time-display vgProperty="current" vgFormat="mm:ss">
</vg-time-display><vg-scrub-bar style="pointer-events: none;"></vg-scrub-bar><vg-time-display vgProperty="left" vgFormat="mm:ss"></vg-time-display>
<vg-time-display vgProperty="total" vgFormat="mm:ss"></vg-time-display><vg-track-selector></vg-track-selector><vg-mute ></vg-mute><vg-volume></vg-volume>
<vg-fullscreen></vg-fullscreen></vg-controls>
<video width="100%" controls disablepictureinpicture controlslist="nodownload" [vgMedia]="$any(media)" #media id= "singleVideo" preload="auto" controls><source src = "`;
const suffix = '"type = video/webm type="video/mp4"> </video></vg-player>';

/*
 * Public API Surface of user-onboarding-lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { TourServiceService, TourStoreServiceService, UserOnboardingLibModule, prefix, suffix };
//# sourceMappingURL=sourceloop-user-onboarding-client.js.map
